# --------------------------------------------------------
# UIBE Cracker
# Licensed under The MIT License
# Written by Ye Liu (goolhanrry at gmail.com)
# --------------------------------------------------------

from io import BytesIO
from time import sleep
from requests import session
import matplotlib.pyplot as plt

API_ROOT = 'http://zhjw.uibe.edu.cn/'


def _get_captcha(sess):
    res = sess.get(API_ROOT + 'verifycode.servlet')
    img = plt.imread(BytesIO(res.content), format='JPEG')
    plt.axis('off')
    plt.imshow(img)
    plt.show()
    return input('CAPTCHA: ')


def _login(sess, username, password):
    captcha = _get_captcha(sess)
    header = {'Content-Type': 'application/x-www-form-urlencoded'}
    query = {'method': 'logon'}
    data = {'USERNAME': username, 'PASSWORD': password, 'RANDOMCODE': captcha}
    res = sess.post(API_ROOT + 'Logon.do',
                    headers=header,
                    params=query,
                    data=data)
    return res.cookies


def _activate_course_mode(sess, mode_id):
    query = {'method': 'comeInXkIndex', 'xklx': mode_id}
    res = sess.get(API_ROOT + 'dwjjmy_jsxsd/xsxkAction.do', params=query)
    return res.status_code == 200


def _search_course(sess, course_id, course_idx, num_per_week):
    query = {
        'method': 'searchKxkc',
        'qKcxx': course_id,
        'qKxh': course_idx,
        'qSkjc': num_per_week,
        'isYm': 'false'
    }
    data = {
        'sEcho': '1',
        'iColumns': '10',
        'iDisplayStart': '0',
        'iDisplayLength': '10'
    }
    res = sess.post(API_ROOT + 'dwjjmy_jsxsd/xsxkAction.do',
                    params=query,
                    data=data)

    try:
        res = res.json()

        if not len(res['aaData']):
            print('Error: Course not found')
            exit()

        if len(res['aaData']) > 1:
            print(res['aaData'])
            print('Error: Multiple courses found, please use tzdid directly')
            exit()

        return res['aaData'][0]['tzdid']
    except Exception:
        print('Error: Search course failed')
        return None


def _book_course(sess, tzdid):
    query = {'method': 'xsxkOper', 'tzdid': tzdid}
    res = sess.get(API_ROOT + 'dwjjmy_jsxsd/xsxkAction.do', params=query)

    try:
        res = res.json()
    except Exception:
        res = {'success': False, 'message': '选课系统未开放'}

    print(res)
    return res['success']


def meow(username,
         password,
         mode_id,
         tzdid=None,
         course_id=None,
         course_idx=None,
         num_per_week=None,
         interval=0.2):
    sess = session()

    while not _login(sess, username, password):
        pass

    if not _activate_course_mode(sess, mode_id):
        print('Error: Activate course mode failed')
        return

    while True:
        if not tzdid:
            tzdid = _search_course(sess, course_id, course_idx, num_per_week)

        if tzdid and _book_course(sess, tzdid):
            break

        sleep(interval)
